@@ -5,24 +5,43 @@
  *
  * @package WordPress
  * @subpackage Multisite
- * @since 3.0.0
+ * @since MU
  */
 
-// @todo use update_blog_details
+/**
+ * Update the last_updated field for the current blog.
+ *
+ * @since MU
+ */
 function wpmu_update_blogs_date() {
 	global $wpdb;
 
-	$wpdb->update( $wpdb->blogs, array('last_updated' => current_time('mysql', true)), array('blog_id' => $wpdb->blogid) );
-	refresh_blog_details( $wpdb->blogid );
+	update_blog_details( $wpdb->blogid, array('last_updated' => current_time('mysql', true)) );
 
 	do_action( 'wpmu_blog_updated', $wpdb->blogid );
 }
 
+/**
+ * Get a full blog URL, given a blog id.
+ *
+ * @since MU
+ *
+ * @param int $blog_id Blog ID
+ * @return string
+ */
 function get_blogaddress_by_id( $blog_id ) {
 	$bloginfo = get_blog_details( (int) $blog_id, false ); // only get bare details!
 	return esc_url( 'http://' . $bloginfo->domain . $bloginfo->path );
 }
 
+/**
+ * Get a full blog URL, given a blog name.
+ *
+ * @since MU
+ *
+ * @param string $blogname The (subdomain or directory) name
+ * @return string
+ */
 function get_blogaddress_by_name( $blogname ) {
 	global $current_site;
 
@@ -38,9 +57,18 @@ function get_blogaddress_by_name( $blogname ) {
 	return esc_url( $url . '/' );
 }
 
-function get_blogaddress_by_domain( $domain, $path ){
+/**
+ * Get a full blog URL, given a domain and a path.
+ *
+ * @since MU
+ *
+ * @param string $domain
+ * @param string $path
+ * @return string
+ */
+function get_blogaddress_by_domain( $domain, $path ) {
 	if ( is_subdomain_install() ) {
-		$url = "http://".$domain.$path;
+		$url = "http://" . $domain.$path;
 	} else {
 		if ( $domain != $_SERVER['HTTP_HOST'] ) {
 			$blogname = substr( $domain, 0, strpos( $domain, '.' ) );
@@ -55,9 +83,17 @@ function get_blogaddress_by_domain( $domain, $path ){
 	return esc_url( $url );
 }
 
+/**
+ * Given a blog's (subdomain or directory) name, retrieve it's id.
+ *
+ * @since MU
+ *
+ * @param string $name
+ * @return int A blog id
+ */
 function get_id_from_blogname( $name ) {
 	global $wpdb, $current_site;
-	$blog_id = wp_cache_get( "get_id_from_blogname_" . $name, 'blog-details' );
+	$blog_id = wp_cache_get( 'get_id_from_blogname_' . $name, 'blog-details' );
 	if ( $blog_id )
 		return $blog_id;
 
@@ -76,7 +112,8 @@ function get_id_from_blogname( $name ) {
 /**
  * Retrieve the details for a blog from the blogs table and blog options.
  *
- * @since 3.0.0
+ * @since MU
+ *
  * @param int|string|array $fields A blog ID, a blog name, or an array of fields to query against.
  * @param bool $get_all Whether to retrieve all details or only the details in the blogs table. Default is true.
  * @return object Blog details.
@@ -202,7 +239,7 @@ function get_blog_details( $fields, $get_all = true ) {
 /**
  * Clear the blog details cache.
  *
- * @since 3.0.0
+ * @since MU
  *
  * @param int $blog_id Blog ID
  */
@@ -215,12 +252,14 @@ function refresh_blog_details( $blog_id ) {
 	wp_cache_delete( md5( $details->domain . $details->path )  , 'blog-lookup' );
 	wp_cache_delete( 'current_blog_' . $details->domain, 'site-options' );
 	wp_cache_delete( 'current_blog_' . $details->domain . $details->path, 'site-options' );
+
+	do_action( 'refresh_blog_details', $blog_id );
 }
 
 /**
  * Update the details for a blog. Updates the blogs table for a given blog id.
  *
- * @since 3.0.0
+ * @since MU
  *
  * @param int $blog_id Blog ID
  * @param array $details Array of details keyed by blogs table field names.
@@ -260,7 +299,7 @@ function update_blog_details( $blog_id, $details = array() ) {
 	}
 
 	if ( isset($details[ 'public' ]) )
-		update_blog_option( $blog_id, 'blog_public', $details[ 'public' ], false );
+		update_blog_option( $blog_id, 'blog_public', $details[ 'public' ] );
 
 	refresh_blog_details($blog_id);
 
@@ -280,21 +319,22 @@ function update_blog_details( $blog_id, $details = array() ) {
  * $blog_id. It returns $value.
  * The 'option_$option' filter in get_option() is not called.
  *
- * @since NA
- * @package WordPress MU
- * @subpackage Option
+ * @since MU
  * @uses apply_filters() Calls 'blog_option_$optionname' with the option name value.
  *
- * @param int $blog_id is the id of the blog.
- * @param string $setting Name of option to retrieve. Should already be SQL-escaped
+ * @param int $blog_id Optional. Blog ID, can be null to refer to the current blog.
+ * @param string $setting Name of option to retrieve. Should already be SQL-escaped.
  * @param string $default (optional) Default value returned if option not found.
  * @return mixed Value set for the option.
  */
 function get_blog_option( $blog_id, $setting, $default = false ) {
 	global $wpdb;
 
-	$key = $blog_id."-".$setting."-blog_option";
-	$value = wp_cache_get( $key, "site-options" );
+	if ( null === $blog_id )
+		$blog_id = $wpdb->blogid;
+
+	$key = $blog_id . '-' . $setting . '-blog_option';
+	$value = wp_cache_get( $key, 'site-options' );
 	if ( $value == null ) {
 		if ( $blog_id == $wpdb->blogid ) {
 			$value = get_option( $setting, $default );
@@ -334,44 +374,96 @@ function get_blog_option( $blog_id, $setting, $default = false ) {
 	if ( 'siteurl' == $setting || 'home' == $setting || 'category_base' == $setting )
 		$value = untrailingslashit( $value );
 
-	if (! @unserialize( $value ) )
-		$value = stripslashes( $value );
-
 	return apply_filters( 'blog_option_' . $setting, maybe_unserialize( $value ), $blog_id );
 }
 
+/**
+ * Add an option for a particular blog.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @param string $key The option key
+ * @param mixed $value The option value
+ * @return bool True on success, false on failure.
+ */
 function add_blog_option( $id, $key, $value ) {
 	$id = (int) $id;
 
 	switch_to_blog($id);
-	add_option( $key, $value );
+	$return = add_option( $key, $value );
 	restore_current_blog();
-	wp_cache_set( $id."-".$key."-blog_option", $value, 'site-options' );
+	if ( $return )
+		wp_cache_set( $id . '-' . $key . '-blog_option', $value, 'site-options' );
+	return $return;
 }
 
+/**
+ * Delete an option for a particular blog.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @param string $key The option key
+ * @return bool True on success, false on failure.
+ */
 function delete_blog_option( $id, $key ) {
 	$id = (int) $id;
 
 	switch_to_blog($id);
-	delete_option( $key );
+	$return = delete_option( $key );
 	restore_current_blog();
-	wp_cache_set( $id."-".$key."-blog_option", '', 'site-options' );
+	if ( $return )
+		wp_cache_set( $id . '-' . $key . '-blog_option', '', 'site-options' );
+	return $return;
 }
 
-function update_blog_option( $id, $key, $value, $refresh = true ) {
+/**
+ * Update an option for a particular blog.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @param string $key The option key
+ * @param mixed $value The option value
+ * @return bool True on success, false on failrue.
+ */
+function update_blog_option( $id, $key, $value, $deprecated = null ) {
 	$id = (int) $id;
 
+	if ( null !== $deprecated  )
+		_deprecated_argument( __FUNCTION__, '3.1' );
+
 	switch_to_blog($id);
-	update_option( $key, $value );
+	$return = update_option( $key, $value );
 	restore_current_blog();
 
-	if ( $refresh == true )
-		refresh_blog_details( $id );
-	wp_cache_set( $id."-".$key."-blog_option", $value, 'site-options');
+	refresh_blog_details( $id );
+
+	if ( $return )
+		wp_cache_set( $id . '-' . $key . '-blog_option', $value, 'site-options');
+	return $return;
 }
 
+/**
+ * Switch the current blog.
+ *
+ * This function is useful if you need to pull posts, or other information,
+ * from other blogs. You can switch back afterwards using restore_current_blog().
+ *
+ * Things that aren't switched:
+ *  - autoloaded options. See #14992
+ *  - plugins. See #14941
+ *
+ * @see restore_current_blog()
+ * @since MU
+ *
+ * @param int $new_blog The id of the blog you want to switch to. Default: current blog
+ * @param bool $validate Whether to check if $new_blog exists before proceeding
+ * @return bool	True on success, False if the validation failed
+ */
 function switch_to_blog( $new_blog, $validate = false ) {
-	global $wpdb, $table_prefix, $blog_id, $switched, $switched_stack, $wp_roles, $current_user, $wp_object_cache;
+	global $wpdb, $table_prefix, $blog_id, $switched, $switched_stack, $wp_roles, $wp_object_cache;
 
 	if ( empty($new_blog) )
 		$new_blog = $blog_id;
@@ -407,8 +499,11 @@ function switch_to_blog( $new_blog, $validate = false ) {
 		$wpdb->suppress_errors( false );
 	}
 
-	if ( is_object( $current_user ) )
-		$current_user->for_blog( $blog_id );
+	if ( did_action('init') ) {
+		$current_user = wp_get_current_user();
+		if ( is_object( $current_user ) )
+			$current_user->for_blog( $blog_id );
+	}
 
 	if ( is_object( $wp_object_cache ) && isset( $wp_object_cache->global_groups ) )
 		$global_groups = $wp_object_cache->global_groups;
@@ -420,7 +515,7 @@ function switch_to_blog( $new_blog, $validate = false ) {
 		if ( is_array( $global_groups ) )
 			wp_cache_add_global_groups( $global_groups );
 		else
-			wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'site-transient', 'global-posts' ) );
+			wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'site-transient', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'global-posts' ) );
 		wp_cache_add_non_persistent_groups(array( 'comment', 'counts', 'plugins' ));
 	}
 
@@ -429,8 +524,16 @@ function switch_to_blog( $new_blog, $validate = false ) {
 	return true;
 }
 
+/**
+ * Restore the current blog, after calling switch_to_blog()
+ *
+ * @see switch_to_blog()
+ * @since MU
+ *
+ * @return bool True on success, False if we're already on the current blog
+ */
 function restore_current_blog() {
-	global $table_prefix, $wpdb, $blog_id, $switched, $switched_stack, $wp_roles, $current_user, $wp_object_cache;
+	global $table_prefix, $wpdb, $blog_id, $switched, $switched_stack, $wp_roles, $wp_object_cache;
 
 	if ( !$switched )
 		return false;
@@ -460,8 +563,11 @@ function restore_current_blog() {
 		$wpdb->suppress_errors( false );
 	}
 
-	if ( is_object( $current_user ) )
-		$current_user->for_blog( $blog_id );
+	if ( did_action('init') ) {
+		$current_user = wp_get_current_user();
+		if ( is_object( $current_user ) )
+			$current_user->for_blog( $blog_id );
+	}
 
 	if ( is_object( $wp_object_cache ) && isset( $wp_object_cache->global_groups ) )
 		$global_groups = $wp_object_cache->global_groups;
@@ -473,7 +579,7 @@ function restore_current_blog() {
 		if ( is_array( $global_groups ) )
 			wp_cache_add_global_groups( $global_groups );
 		else
-			wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'site-transient' ) );
+			wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'site-transient', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'global-posts' ) );
 		wp_cache_add_non_persistent_groups(array( 'comment', 'counts', 'plugins' ));
 	}
 
@@ -484,10 +590,27 @@ function restore_current_blog() {
 	return true;
 }
 
+/**
+ * Check if a particular blog is archived.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @return string Whether the blog is archived or not
+ */
 function is_archived( $id ) {
 	return get_blog_status($id, 'archived');
 }
 
+/**
+ * Update the 'archived' status of a particular blog.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @param string $archived The new status
+ * @return string $archived
+ */
 function update_archived( $id, $archived ) {
 	update_blog_status($id, 'archived', $archived);
 	return $archived;
@@ -496,34 +619,47 @@ function update_archived( $id, $archived ) {
 /**
  * Update a blog details field.
  *
- * @since 3.0.0
+ * @since MU
  *
  * @param int $blog_id BLog ID
  * @param string $pref A field name
  * @param string $value Value for $pref
- * @param bool $refresh Whether to refresh the blog details cache. Default is true.
+ * @return string $value
  */
-function update_blog_status( $blog_id, $pref, $value, $refresh = true ) {
+function update_blog_status( $blog_id, $pref, $value, $deprecated = null ) {
 	global $wpdb;
 
+	if ( null !== $deprecated  )
+		_deprecated_argument( __FUNCTION__, '3.1' );
+
 	if ( !in_array( $pref, array( 'site_id', 'domain', 'path', 'registered', 'last_updated', 'public', 'archived', 'mature', 'spam', 'deleted', 'lang_id') ) )
 		return $value;
 
 	$wpdb->update( $wpdb->blogs, array($pref => $value, 'last_updated' => current_time('mysql', true)), array('blog_id' => $blog_id) );
 
-	if ( $refresh )
-		refresh_blog_details($blog_id);
+	refresh_blog_details($blog_id);
 
-	if ( $pref == 'spam' ) {
-		if ( $value == 1 )
-			do_action( "make_spam_blog", $blog_id );
-		else
-			do_action( "make_ham_blog", $blog_id );
-	}
+	if ( 'spam' == $pref )
+		( $value == 1 ) ? do_action( 'make_spam_blog', $blog_id ) :	do_action( 'make_ham_blog', $blog_id );
+	elseif ( 'mature' == $pref )
+		( $value == 1 ) ? do_action( 'mature_blog', $blog_id ) : do_action( 'unmature_blog', $blog_id );
+	elseif ( 'archived' == $pref )
+		( $value == 1 ) ? do_action( 'archive_blog', $blog_id ) : do_action( 'unarchive_blog', $blog_id );
+	elseif ( 'archived' == $pref )
+		( $value == 1 ) ? do_action( 'archive_blog', $blog_id ) : do_action( 'unarchive_blog', $blog_id );
 
 	return $value;
 }
 
+/**
+ * Get a blog details field.
+ *
+ * @since MU
+ *
+ * @param int $id The blog id
+ * @param string $pref A field name
+ * @return bool $value
+ */
 function get_blog_status( $id, $pref ) {
 	global $wpdb;
 
@@ -534,9 +670,64 @@ function get_blog_status( $id, $pref ) {
 	return $wpdb->get_var( $wpdb->prepare("SELECT %s FROM {$wpdb->blogs} WHERE blog_id = %d", $pref, $id) );
 }
 
+/**
+ * Get a list of most recently updated blogs.
+ *
+ * @since MU
+ *
+ * @param mixed $deprecated Not used
+ * @param int $start The offset
+ * @param int $quantity The maximum number of blogs to retrieve. Default is 40.
+ * @return array The list of blogs
+ */
 function get_last_updated( $deprecated = '', $start = 0, $quantity = 40 ) {
 	global $wpdb;
+
+	if ( ! empty( $deprecated ) )
+		_deprecated_argument( __FUNCTION__, 'MU' ); // never used
+
 	return $wpdb->get_results( $wpdb->prepare("SELECT blog_id, domain, path FROM $wpdb->blogs WHERE site_id = %d AND public = '1' AND archived = '0' AND mature = '0' AND spam = '0' AND deleted = '0' AND last_updated != '0000-00-00 00:00:00' ORDER BY last_updated DESC limit %d, %d", $wpdb->siteid, $start, $quantity ) , ARRAY_A );
 }
 
-?>
+/**
+ * Handler for updating the blog date when a post is published or an already published post is changed.
+ *
+ * @since 3.3.0
+ *
+ * @param string $new_status The new post status
+ * @param string $old_status The old post status
+ * @param object $post Post object
+ */
+function _update_blog_date_on_post_publish( $new_status, $old_status, $post ) {
+	$post_type_obj = get_post_type_object( $post->post_type );
+	if ( ! $post_type_obj->public )
+		return;
+
+	if ( 'publish' != $new_status && 'publish' != $old_status )
+		return;
+
+	// Post was freshly published, published post was saved, or published post was unpublished.
+
+	wpmu_update_blogs_date();
+}
+
+/**
+ * Handler for updating the blog date when a published post is deleted.
+ *
+ * @since 3.4.0
+ *
+ * @param int $post_id Post ID
+ */
+function _update_blog_date_on_post_delete( $post_id ) {
+	$post = get_post( $post_id );
+
+	$post_type_obj = get_post_type_object( $post->post_type );
+	if ( ! $post_type_obj->public )
+		return;
+
+	if ( 'publish' != $post->post_status )
+		return;
+
+	wpmu_update_blogs_date();
+}
+
