@@ -79,7 +79,6 @@ function the_title_attribute( $args = '' ) {
 	$r = wp_parse_args($args, $defaults);
 	extract( $r, EXTR_SKIP );
 
-
 	$title = $before . $title . $after;
 	$title = esc_attr(strip_tags($title));
 
@@ -188,10 +187,8 @@ function get_the_content($more_link_text = null, $stripteaser = false) {
 	$hasTeaser = false;
 
 	// If post password required and it doesn't match the cookie.
-	if ( post_password_required($post) ) {
-		$output = get_the_password_form();
-		return $output;
-	}
+	if ( post_password_required($post) )
+		return get_the_password_form();
 
 	if ( $page > count($pages) ) // if the requested page doesn't exist
 		$page = count($pages); // give them the highest numbered page that DOES exist
@@ -414,8 +411,10 @@ function get_body_class( $class = '' ) {
 		$classes[] = 'archive';
 	if ( is_date() )
 		$classes[] = 'date';
-	if ( is_search() )
+	if ( is_search() ) {
 		$classes[] = 'search';
+		$classes[] = $wp_query->posts ? 'search-results' : 'search-no-results';
+	}
 	if ( is_paged() )
 		$classes[] = 'paged';
 	if ( is_attachment() )
@@ -490,15 +489,10 @@ function get_body_class( $class = '' ) {
 		}
 		if ( is_page_template() ) {
 			$classes[] = 'page-template';
-			$classes[] = 'page-template-' . sanitize_html_class( str_replace( '.', '-', get_post_meta( $page_id, '_wp_page_template', true ) ), '' );
+			$classes[] = 'page-template-' . sanitize_html_class( str_replace( '.', '-', get_page_template_slug( $page_id ) ) );
 		} else {
 			$classes[] = 'page-template-default';
 		}
-	} elseif ( is_search() ) {
-		if ( !empty( $wp_query->posts ) )
-			$classes[] = 'search-results';
-		else
-			$classes[] = 'search-no-results';
 	}
 
 	if ( is_user_logged_in() )
@@ -507,7 +501,7 @@ function get_body_class( $class = '' ) {
 	if ( is_admin_bar_showing() )
 		$classes[] = 'admin-bar';
 
-	if ( get_background_image() || get_background_color() )
+	if ( get_theme_mod( 'background_color' ) || get_background_image() )
 		$classes[] = 'custom-background';
 
 	$page = $wp_query->get( 'page' );
@@ -555,22 +549,29 @@ function get_body_class( $class = '' ) {
  *
  * @since 2.7.0
  *
- * @param int|object $post An optional post.  Global $post used if not provided.
+ * @param int|object $post An optional post. Global $post used if not provided.
  * @return bool false if a password is not required or the correct password cookie is present, true otherwise.
  */
 function post_password_required( $post = null ) {
+	global $wp_hasher;
+
 	$post = get_post($post);
 
-	if ( empty($post->post_password) )
+	if ( empty( $post->post_password ) )
 		return false;
 
-	if ( !isset($_COOKIE['wp-postpass_' . COOKIEHASH]) )
+	if ( ! isset( $_COOKIE['wp-postpass_' . COOKIEHASH] ) )
 		return true;
 
-	if ( stripslashes( $_COOKIE['wp-postpass_' . COOKIEHASH] ) != $post->post_password )
-		return true;
+	if ( empty( $wp_hasher ) ) {
+		require_once( ABSPATH . 'wp-includes/class-phpass.php');
+		// By default, use the portable hash from phpass
+		$wp_hasher = new PasswordHash(8, true);
+	}
 
-	return false;
+	$hash = stripslashes( $_COOKIE[ 'wp-postpass_' . COOKIEHASH ] );
+
+	return ! $wp_hasher->CheckPassword( $post->post_password, $hash );
 }
 
 /**
@@ -994,7 +995,7 @@ class Walker_Page extends Walker {
 	 * @param string $output Passed by reference. Used to append additional content.
 	 * @param int $depth Depth of page. Used for padding.
 	 */
-	function start_lvl(&$output, $depth) {
+	function start_lvl( &$output, $depth = 0, $args = array() ) {
 		$indent = str_repeat("\t", $depth);
 		$output .= "\n$indent<ul class='children'>\n";
 	}
@@ -1006,7 +1007,7 @@ class Walker_Page extends Walker {
 	 * @param string $output Passed by reference. Used to append additional content.
 	 * @param int $depth Depth of page. Used for padding.
 	 */
-	function end_lvl(&$output, $depth) {
+	function end_lvl( &$output, $depth = 0, $args = array() ) {
 		$indent = str_repeat("\t", $depth);
 		$output .= "$indent</ul>\n";
 	}
@@ -1021,7 +1022,7 @@ class Walker_Page extends Walker {
 	 * @param int $current_page Page ID.
 	 * @param array $args
 	 */
-	function start_el(&$output, $page, $depth, $args, $current_page) {
+	function start_el( &$output, $page, $depth, $args, $current_page = 0 ) {
 		if ( $depth )
 			$indent = str_repeat("\t", $depth);
 		else
@@ -1064,7 +1065,7 @@ class Walker_Page extends Walker {
 	 * @param object $page Page data object. Not used.
 	 * @param int $depth Depth of page. Not Used.
 	 */
-	function end_el(&$output, $page, $depth) {
+	function end_el( &$output, $page, $depth = 0, $args = array() ) {
 		$output .= "</li>\n";
 	}
 
@@ -1102,7 +1103,7 @@ class Walker_PageDropdown extends Walker {
 	 * @param int $depth Depth of page in reference to parent pages. Used for padding.
 	 * @param array $args Uses 'selected' argument for selected page to set selected HTML attribute for option element.
 	 */
-	function start_el(&$output, $page, $depth, $args) {
+	function start_el(&$output, $page, $depth, $args, $id = 0) {
 		$pad = str_repeat('&nbsp;', $depth * 3);
 
 		$output .= "\t<option class=\"level-$depth\" value=\"$page->ID\"";
@@ -1165,7 +1166,7 @@ function wp_get_attachment_link( $id = 0, $size = 'thumbnail', $permalink = fals
 	$post_title = esc_attr( $_post->post_title );
 
 	if ( $text )
-		$link_text = esc_attr( $text );
+		$link_text = $text;
 	elseif ( $size && 'none' != $size )
 		$link_text = wp_get_attachment_image( $id, $size, $icon );
 	else
@@ -1215,11 +1216,11 @@ function prepend_attachment($content) {
  */
 function get_the_password_form() {
 	global $post;
-	$label = 'pwbox-'.(empty($post->ID) ? rand() : $post->ID);
-	$output = '<form action="' . get_option('siteurl') . '/wp-pass.php" method="post">
+	$label = 'pwbox-' . ( empty($post->ID) ? rand() : $post->ID );
+	$output = '<form action="' . esc_url( site_url( 'wp-login.php?action=postpass', 'login_post' ) ) . '" method="post">
 	<p>' . __("This post is password protected. To view it please enter your password below:") . '</p>
 	<p><label for="' . $label . '">' . __("Password:") . ' <input name="post_password" id="' . $label . '" type="password" size="20" /></label> <input type="submit" name="Submit" value="' . esc_attr__("Submit") . '" /></p>
-	</form>
+</form>
 	';
 	return apply_filters('the_password_form', $output);
 }
@@ -1237,29 +1238,43 @@ function get_the_password_form() {
  * @param string $template The specific template name if specific matching is required.
  * @return bool False on failure, true if success.
  */
-function is_page_template($template = '') {
-	if (!is_page()) {
+function is_page_template( $template = '' ) {
+	if ( ! is_page() )
 		return false;
-	}
 
-	global $wp_query;
+	$page_template = get_page_template_slug( get_queried_object_id() );
 
-	$page = $wp_query->get_queried_object();
-	$custom_fields = get_post_custom_values('_wp_page_template',$page->ID);
-	$page_template = $custom_fields[0];
+	if ( empty( $template ) )
+		return (bool) $page_template;
 
-	// We have no argument passed so just see if a page_template has been specified
-	if ( empty( $template ) ) {
-		if ( !empty( $page_template ) and ( 'default' != $page_template ) ) {
-			return true;
-		}
-	} elseif ( $template == $page_template) {
+	if ( $template == $page_template )
+		return true;
+
+	if ( 'default' == $template && ! $page_template )
 		return true;
-	}
 
 	return false;
 }
 
+/**
+ * Get the specific template name for a page.
+ *
+ * @since 3.4.0
+ *
+ * @param int $id The page ID to check. Defaults to the current post, when used in the loop.
+ * @return string|bool Page template filename. Returns an empty string when the default page template
+ * 	is in use. Returns false if the post is not a page.
+ */
+function get_page_template_slug( $post_id = null ) {
+	$post = get_post( $post_id );
+	if ( 'page' != $post->post_type )
+		return false;
+	$template = get_post_meta( $post->ID, '_wp_page_template', true );
+	if ( ! $template || 'default' == $template )
+		return '';
+	return $template;
+}
+
 /**
  * Retrieve formatted date timestamp of a revision (linked to that revisions's page).
  *
@@ -1307,7 +1322,7 @@ function wp_post_revision_title( $revision, $link = true ) {
  *
  * Second argument controls parameters:
  *   (bool)   parent : include the parent (the "Current Revision") in the list.
- *   (string) format : 'list' or 'form-table'.  'list' outputs UL, 'form-table'
+ *   (string) format : 'list' or 'form-table'. 'list' outputs UL, 'form-table'
  *                     outputs TABLE with UI.
  *   (int)    right  : what revision is currently being viewed - used in
  *                     form-table format.
